package com.icingDialer

import io.flutter.embedding.android.FlutterActivity
import android.os.Build
import android.security.keystore.KeyGenParameterSpec
import android.security.keystore.KeyProperties
import androidx.annotation.NonNull
import io.flutter.embedding.android.FlutterActivity
import io.flutter.embedding.engine.FlutterEngine
import io.flutter.plugin.common.MethodChannel
import java.security.KeyStore
import javax.crypto.KeyGenerator
import javax.crypto.SecretKey
import android.util.Base64

class MainActivity: FlutterActivity() {
    private val CHANNEL = "com.yourapp/keystore"

    override fun configureFlutterEngine(@NonNull flutterEngine: FlutterEngine) {
        super.configureFlutterEngine(flutterEngine)
        MethodChannel(flutterEngine.dartExecutor.binaryMessenger, CHANNEL).setMethodCallHandler { call, result ->
            when (call.method) {
                "getSymmetricKey" -> {
                    try {
                        getOrCreateSymmetricKey()
                        result.success("symmetric_key_alias")
                    } catch (e: Exception) {
                        result.error("KEY_ERROR", e.message, null)
                    }
                }
                "encryptSeed" -> {
                    val args = call.arguments as Map<String, String>
                    val seed = args["seed"]?.let { Base64.decode(it, Base64.DEFAULT) }
                    if (seed == null) {
                        result.error("INVALID_ARGUMENT", "Seed is null", null)
                        return@setMethodCallHandler
                    }
                    try {
                        val encrypted = encryptSeed(seed)
                        result.success(Base64.encodeToString(encrypted, Base64.DEFAULT))
                    } catch (e: Exception) {
                        result.error("ENCRYPTION_ERROR", e.message, null)
                    }
                }
                "decryptSeed" -> {
                    val args = call.arguments as Map<String, String>
                    val encryptedSeed = args["encryptedSeed"]?.let { Base64.decode(it, Base64.DEFAULT) }
                    if (encryptedSeed == null) {
                        result.error("INVALID_ARGUMENT", "Encrypted seed is null", null)
                        return@setMethodCallHandler
                    }
                    try {
                        val decrypted = decryptSeed(encryptedSeed)
                        result.success(Base64.encodeToString(decrypted, Base64.DEFAULT))
                    } catch (e: Exception) {
                        result.error("DECRYPTION_ERROR", e.message, null)
                    }
                }
                else -> {
                    result.notImplemented()
                }
            }
        }
    }

    private fun getOrCreateSymmetricKey(): SecretKey {
        val keyStore = KeyStore.getInstance("AndroidKeyStore").apply { load(null) }
        val alias = "symmetric_key_alias"

        // Check if the key already exists
        if (!keyStore.containsAlias(alias)) {
            // Create the key if it doesn't exist
            val keyGenerator = KeyGenerator.getInstance(KeyProperties.KEY_ALGORITHM_AES, "AndroidKeyStore")
            val keyGenParameterSpec = KeyGenParameterSpec.Builder(
                alias,
                KeyProperties.PURPOSE_ENCRYPT or KeyProperties.PURPOSE_DECRYPT
            )
                .setBlockModes(KeyProperties.BLOCK_MODE_GCM)
                .setEncryptionPaddings(KeyProperties.ENCRYPTION_PADDING_NONE)
                .setKeySize(256)
                .build()
            keyGenerator.init(keyGenParameterSpec)
            return keyGenerator.generateKey()
        }

        // Retrieve the existing key
        return keyStore.getKey(alias, null) as SecretKey
    }

    private fun encryptSeed(seed: ByteArray): ByteArray {
        val keyStore = KeyStore.getInstance("AndroidKeyStore").apply { load(null) }
        val key = keyStore.getKey("symmetric_key_alias", null) as SecretKey

        val cipher = Cipher.getInstance("AES/GCM/NoPadding")
        cipher.init(Cipher.ENCRYPT_MODE, key)
        val encryptionIv = cipher.iv
        val encryptedBytes = cipher.doFinal(seed)

        // Prepend IV to encrypted bytes for storage
        return encryptionIv + encryptedBytes
    }

    private fun decryptSeed(encryptedSeed: ByteArray): ByteArray {
        val keyStore = KeyStore.getInstance("AndroidKeyStore").apply { load(null) }
        val key = keyStore.getKey("symmetric_key_alias", null) as SecretKey

        // Extract IV and encrypted bytes
        val iv = encryptedSeed.copyOfRange(0, 12) // GCM IV is 12 bytes
        val ciphertext = encryptedSeed.copyOfRange(12, encryptedSeed.size)

        val cipher = Cipher.getInstance("AES/GCM/NoPadding")
        cipher.init(Cipher.DECRYPT_MODE, key, javax.crypto.spec.GCMParameterSpec(128, iv))
        return cipher.doFinal(ciphertext)
    }
}
